// Controller.m
//

#import "Controller.h"
#import "AGProcess.h"

NSString *stringForEnvironment(NSDictionary *env) {
    NSMutableString *str = [NSMutableString string];
    NSEnumerator *en = [env keyEnumerator];
    id key;
    while (key = [en nextObject])
        [str appendFormat:@"%@ = %@, ", key, [env objectForKey:key]];
    return str;
}
    
NSString *stringForState(int s) {
    switch (s) {
	case AGProcessStateUnknown: return @"";
	case AGProcessStateRunnable: return @"Runnable";
	case AGProcessStateUninterruptible: return @"Uninterruptible";
	case AGProcessStateSleeping: return @"Sleeping";
	case AGProcessStateIdle: return @"Idle";
	case AGProcessStateSuspended: return @"Suspended";
	case AGProcessStateZombie: return @"Zombie";
	case AGProcessStateExited: return @"Exited";
    }
}

@implementation Controller

- (void)awakeFromNib {
    refreshStatisticsTimer = [[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(refreshStatistics:) userInfo:nil repeats:YES] retain];
}

- (void)refreshStatistics:(id)sender {
    if ([[[browser selectedCell] representedObject] isKindOfClass:[AGProcess class]]) {
        AGProcess *process = [[browser selectedCell] representedObject];
        
        [commandCell setStringValue:[process command]];
        [argumentsCell setStringValue:[[process arguments] componentsJoinedByString:@" "]];    
        [environmentCell setStringValue:stringForEnvironment([process environment])];     
        [processIdentifierCell setIntValue:[process processIdentifier]];
        [parentProcessIdentifierCell setIntValue:[process parentProcessIdentifier]];
        [processGroupCell setIntValue:[process processGroup]];
        [userIdentifierCell setIntValue:[process userIdentifier]];
        [realUserIdentifierCell setIntValue:[process realUserIdentifier]];
        [terminalCell setIntValue:[process terminal]];
        [terminalProcessGroupCell setIntValue:[process terminalProcessGroup]];
        [percentCPUCell setFloatValue:[process percentCPUUsage] * 100];
        [totalCPUCell setFloatValue:[process totalCPUTime]];
        [userCPUCell setFloatValue:[process userCPUTime]];
        [systemCPUCell setFloatValue:[process systemCPUTime]];
        [percentMemoryCell setFloatValue:[process percentMemoryUsage] * 100];
        [residentMemoryCell setFloatValue:[process residentMemorySize] / (float)1048576];
        [virtualMemoryCell setFloatValue:[process virtualMemorySize] / (float)1048576];
        [stateCell setStringValue:stringForState([process state])];
        [priorityCell setStringValue:[NSString stringWithFormat:@"%d/%d", [process priority], [process basePriority]]];
        [threadsCell setIntValue:[process threadCount]];
    }
    else {
        [commandCell setStringValue:@""];
        [argumentsCell setStringValue:@""];    
        [environmentCell setStringValue:@""];     
        [processIdentifierCell setStringValue:@""];
        [parentProcessIdentifierCell setStringValue:@""];
        [processGroupCell setStringValue:@""];
        [terminalCell setStringValue:@""];
        [terminalProcessGroupCell setStringValue:@""];
        [userIdentifierCell setStringValue:@""];
        [realUserIdentifierCell setStringValue:@""];
        [percentCPUCell setStringValue:@""];
        [totalCPUCell setStringValue:@""];
        [userCPUCell setStringValue:@""];
        [systemCPUCell setStringValue:@""];
        [percentMemoryCell setStringValue:@""];
        [residentMemoryCell setStringValue:@""];
        [virtualMemoryCell setStringValue:@""];
        [stateCell setStringValue:@""];
        [priorityCell setStringValue:@""];
        [threadsCell setStringValue:@""];
    }

}


@end

@implementation Controller (BrowserDelegate)

- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column {
   	if (column == 0)
        return 3;
    id representedObject = [[sender selectedCellInColumn:column - 1] representedObject];
    if ([representedObject isKindOfClass:[NSArray class]])
        return [representedObject count];
    else
        return 3;
}

- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column {
    
    if (column == 0) {
        switch (row) {
        case 0:
            [cell setStringValue:@"Current Process"];
            [cell setRepresentedObject:[NSArray arrayWithObject:[AGProcess currentProcess]]];
            break;
        case 1:
            [cell setStringValue:@"All Processes"];
            [cell setRepresentedObject:[AGProcess allProcesses]];
            break;
        case 2:
            [cell setStringValue:@"User Processes"];
            [cell setRepresentedObject:[AGProcess userProcesses]];
            break;
        }
    }
    else if (column % 2 != 0) {
        AGProcess *process = [[[sender selectedCellInColumn:column - 1] representedObject] objectAtIndex:row];
        [cell setStringValue:[process command]];
        [cell setRepresentedObject:process];
    }
    else {
        AGProcess *process = [[sender selectedCellInColumn:column - 1] representedObject];
        switch (row) {
        case 0:
            [cell setStringValue:@"Parent"];
            [cell setRepresentedObject:[NSArray arrayWithObject:[process parent]]];
            break;
        case 1:
            [cell setStringValue:@"Children"];
            [cell setRepresentedObject:[process children]];
            break;
        case 2:
            [cell setStringValue:@"Siblings"];
            [cell setRepresentedObject:[process siblings]];
            break;
        }
    }

}

- (BOOL)browser:(NSBrowser *)sender selectRow:(int)row inColumn:(int)column {
    [self refreshStatistics:self];
    return YES;
}

@end
